t#pragma rtGlobals=1		// Use modern global access method.

// version 2012_01_27
//
// fit parameter name:	dsf_text
// fit startup coef:		dsf_coef
// fit hold string coef:	dsf_hold
// fit constraints:		dsf_cnstr
// all parameters are shown in "DS"+datafolder table
// fit results are shown in "RT"+datafolder table
//
// usage:
// extract free profile
// edit coef, providing reasonable guess for coefficients (use marquee tools)
// fit profile and update parameters if the fit is successful
// fit waterfall
// extract results
//
// does not require DS XOP to run.

Menu "CurveFit"
"Initialize_Fit"
"NormalizeSpectrum"
"RemoveShirley"
"Fit_Single_Profile"
"Fit_WF_Series"
"Flatten"
"Extract_Fit_results"
"Find_Dirac_point"
"Extract_Sigmoid_Results"
"Calculate_Fit_Integrals"
"Waves_to_Binding_Energy"
End

Proc Initialize_Fit()
	FIT_Initialise()
end

Proc NormalizeSpectrum()
	FIT_normalizeSpectrum()
end

Proc RemoveShirley()
	FIT_removeShirleyBackground()
End

Proc Fit_Single_Profile()
	FIT_fitProfile()
end

Proc Fit_WF_Series()
	FIT_fitSeries()
end

Proc Flatten()
	FIT_flattenAll()
end

Proc Extract_Fit_Results()
	FIT_extract_results()
end

proc Find_Dirac_point()
	FIT_findDiracPoint()
end

Proc Extract_Sigmoid_Results()
	FIT_extract_sigmoid_results()
end

Proc Calculate_Fit_Integrals()
	FIT_Calculate_Fit_Integrals()
end

Proc Remove_peak()
	FIT_remove_peak()
end

Proc Waves_to_Binding_Energy()
	FIT_AllWavesToBinding()
end

Function FIT_Initialise()
	// data folder
	String dfStr=SIS_getDataFolderName()
	SetDataFolder root:$(dfStr)
	String tableStr="DS"+dfStr
	// init coef waves
	variable m=100
	make/n=(m)/o dsf_coef=0,dsf_hold=1, dsf_error=NaN
	make/n=(m)/o/T dsf_text=""
	make/n=(m)/o/T dsf_cnstr=""
	String/G holdStr
	SIS_tableToFront(tableStr,"dsf_text")	
	SIS_tableToFront(tableStr,"dsf_coef")	
	SIS_tableToFront(tableStr,"dsf_hold")	
	SIS_tableToFront(tableStr,"dsf_cnstr")
	SIS_tableToFront(tableStr,"dsf_error")
	// variable G
	variable/G fittype=1
	variable/G backgroundtype=1	
End

Function FIT_guessBackground(): GraphMarquee 
	String ctrlname
	// data folder and stack
	String dfStr=SIS_getDataFolderName()
	SetDataFolder root:$(dfStr)
	//
	wave/T dsf_text=root:$(dfStr):dsf_text
	wave dsf_coef=root:$(dfStr):dsf_coef
	wave dsf_hold=root:$(dfStr):dsf_hold
	wave dsf_error=root:$(dfStr):dsf_error
	wave/T dsf_cnstr=root:$(dfStr):dsf_cnstr
	variable c1,c2,c3,c4,h1,h2,h3,h4
	GetMarquee/Z/K left, bottom
	if (V_flag==1)
		c1=0.5*(V_top+V_bottom) 
		Prompt c1, "additive constant"
		h1=1
		Prompt h1, "hold", popup "no;yes;"
		c2=0
		Prompt c2, "slope"
		h2=0
		Prompt h2, "hold", popup "no;yes;"
		c3=0 
		Prompt c3, "quadratic term"
		h3=2
		Prompt h3, "hold", popup "no;yes;"
		c4=0 
		Prompt c4, "cubic term"
		h4=2
		Prompt h4, "hold", popup "no;yes;"		
		DoPrompt "D-S: polynomial background input", c1,h1,c2,h2,c3,h3,c4,h4
		dsf_coef[1]=c1
		dsf_coef[2]=c2
		dsf_coef[3]=c3
		dsf_coef[4]=c4
		dsf_hold[1]=h1-1
		dsf_hold[2]=h2-1
		dsf_hold[3]=h3-1
		dsf_hold[4]=h4-1
		dsf_text[0]="number of peaks"
		dsf_text[1]="constant bkg"	
		dsf_text[2]="linear bkg"	
		dsf_text[3]="quadratic bkg"	
		dsf_text[4]="cubic bkg"	
	endif
End

Function FIT_addStepFunction(): GraphMarquee
	String ctrlname
	// data folder and stack
	String dfStr=SIS_getDataFolderName()
	SetDataFolder root:$(dfStr)
	//
	wave/T dsf_text=root:$(dfStr):dsf_text
	wave dsf_coef=root:$(dfStr):dsf_coef
	wave dsf_hold=root:$(dfStr):dsf_hold
	wave dsf_error=root:$(dfStr):dsf_error
	wave/T dsf_cnstr=root:$(dfStr):dsf_cnstr
	variable dsf_npeaks=dsf_coef[0]
	String tmpHoldStr=""
	variable c1,c2,c3,c4,h1,h2,h3,h4
	GetMarquee/Z/K left, bottom
	// input	
	if (V_flag==1)
		if (dsf_npeaks>=0)
			// position
			c1=(V_left+V_right) /2
			Prompt c1, "positon"
			h1=1
			Prompt h1, "hold", popup "no;yes;"
			// fwhm 
			c2=0.5*abs(-V_left+V_right) 
			Prompt c2, "fwhm"
			h2=1
			Prompt h2, "hold", popup "no;yes;"
			//  amplitude
			c3=abs(V_top-V_bottom) 
			Prompt c3, "amplitude"
			h3=1
			Prompt h3, "hold", popup "no;yes;"
			DoPrompt "StepFunction: input", c1,h1,c2,h2,c3,h3
			if (V_flag==0)
				redimension/n=(8) dsf_coef,dsf_hold,dsf_text,dsf_cnstr,dsf_error
				// stores into coef waves
				dsf_coef[0]=1
				dsf_coef[5]=c1
				dsf_coef[6]=c2 
				dsf_coef[7]=c3 
				dsf_hold[5]=h1-1
				dsf_hold[6]=h2-1
				dsf_hold[7]=h3-1
				dsf_text[5]="position"
				dsf_text[6]="fwhm"	
				dsf_text[7]="height"	
				dsf_cnstr[6]="K6 > 0"		
			endif
		endif
	endif
End

Function FIT_addPeak(): GraphMarquee 
	String ctrlname
	// data folder
	String dfStr=SIS_getDataFolderName()
	SetDataFolder root:$(dfStr)
	//
	wave/T dsf_text=root:$(dfStr):dsf_text
	wave dsf_coef=root:$(dfStr):dsf_coef
	wave dsf_hold=root:$(dfStr):dsf_hold
	wave dsf_error=root:$(dfStr):dsf_error
	wave/T dsf_cnstr=root:$(dfStr):dsf_cnstr
	variable dsf_npeaks=dsf_coef[0]
	String tmpHoldStr=""
	variable c1,c2,c3,c4,c5,h1,h2,h3,h4,h5
	GetMarquee/Z/K left, bottom
	if (V_flag==1)
		if (dsf_npeaks==0)
			c1=0.1
			// Lorentzian width Gamma_L
			Prompt c1, "Lorentzian width"
			h1=2
			Prompt h1, "hold", popup "no;yes;"
			// Asymmetry alpha
			c2=0.1
			Prompt c2, "alpha (asimmetry)"
			h2=2
			Prompt h2, "hold", popup "no;yes;"
			//  Gaussian Width
			c3=0.5*abs(-V_left+V_right) 
			Prompt c3, "Gaussian width (eV)"
			h3=1
			Prompt h3, "hold", popup "no;yes;"
			// Intensity
			c4=(V_top-V_bottom)
			Prompt c4, "peak amplitude (eV)"
			h4=1
			Prompt h4, "hold", popup "no;yes;"
			// Position
			c5=0.5*(V_left+V_right) 
			Prompt c5, "peak position (eV)"
			h5=1
			Prompt h5, "hold", popup "no;yes;"
			DoPrompt "Doniach Sunjic: 1st peak input", c1,h1,c2,h2,c3,h3,c4,h4,c5,h5
			if (V_flag==0)
				redimension/n=((dsf_npeaks+2)*5) dsf_coef,dsf_hold,dsf_text,dsf_cnstr,dsf_error
				// stores into coef waves
				dsf_coef[5]=c1
				dsf_coef[6]=c2 
				dsf_coef[7]=c3 
				dsf_coef[8]=c4 
				dsf_coef[9]=c5
				dsf_hold[5]=h1-1
				dsf_hold[6]=h2-1
				dsf_hold[7]=h3-1
				dsf_hold[8]=h4-1
				dsf_hold[9]=h5-1
				dsf_text[5]="Lorentzian width"
				dsf_text[6]="asymmetry"	
				dsf_text[7]="Gaussian width"	
				dsf_text[8]="Amplitude"	
				dsf_text[9]="Position"
				dsf_cnstr[5]="K5 > 0"	
				dsf_cnstr[6]="K6 > 0"		
				dsf_cnstr[7]="K7 > 0"	
				dsf_cnstr[8]="K8 > 0"
				dsf_cnstr[9]=""
				dsf_npeaks+=1
				dsf_coef[0]+=1
			endif
		else
			// Lorentzian width Gamma_L
			c1=dsf_coef[dsf_npeaks*5]
			Prompt c1, "Lorentzian width"
			h1=dsf_hold[dsf_npeaks*5]+1
			Prompt h1, "hold", popup "no;yes;"
			// Asymmetry alpha
			c2=dsf_coef[dsf_npeaks*5+1]
			Prompt c2, "alpha (asimmetry)"
			h2=dsf_hold[dsf_npeaks*5+1]+1
			Prompt h2, "hold", popup "no;yes;"
			//  Gaussian Width
			c3=0.5*abs(-V_left+V_right) 
			Prompt c3, "Gaussian width (eV)"
			h3=dsf_hold[dsf_npeaks*5+2]+1
			Prompt h3, "hold", popup "no;yes;"
			// Intensity
			c4=(V_top-V_bottom)
			Prompt c4, "peak amplitude (eV)"
			h4=dsf_hold[dsf_npeaks*5+3]+1
			Prompt h4, "hold", popup "no;yes;"
			// Position
			c5=0.5*(V_left+V_right) 
			Prompt c5, "peak position (eV)"
			h5=dsf_hold[dsf_npeaks*5+4]+1
			Prompt h5, "hold", popup "no;yes;"
			DoPrompt "Doniach Sunjic: i-th peak input", c1,h1,c2,h2,c3,h3,c4,h4,c5,h5
			if (V_flag==0)
				redimension/n=((dsf_npeaks+2)*5) dsf_coef,dsf_hold,dsf_text,dsf_cnstr,dsf_error
				// stores coef
				dsf_coef[(dsf_npeaks+1)*5]=c1
				dsf_coef[(dsf_npeaks+1)*5+1]=c2 
				dsf_coef[(dsf_npeaks+1)*5+2]=c3 
				dsf_coef[(dsf_npeaks+1)*5+3]=c4 
				dsf_coef[(dsf_npeaks+1)*5+4]=c5
				dsf_hold[(dsf_npeaks+1)*5]=h1-1
				dsf_hold[(dsf_npeaks+1)*5+1]=h2-1 
				dsf_hold[(dsf_npeaks+1)*5+2]=h3-1 
				dsf_hold[(dsf_npeaks+1)*5+3]=h4-1 
				dsf_hold[(dsf_npeaks+1)*5+4]=h5-1
				dsf_text[(dsf_npeaks+1)*5]="Lorentzian width"
				dsf_text[(dsf_npeaks+1)*5+1]="asymmetry"	
				dsf_text[(dsf_npeaks+1)*5+2]="Gaussian width"	
				dsf_text[(dsf_npeaks+1)*5+3]="Amplitude"	
				dsf_text[(dsf_npeaks+1)*5+4]="Position"
				dsf_cnstr[(dsf_npeaks+1)*5]="K"+num2str((dsf_npeaks+1)*5)+" > 0"
				dsf_cnstr[(dsf_npeaks+1)*5+1]="K"	+num2str((dsf_npeaks+1)*5+1)+" > 0"	
				dsf_cnstr[(dsf_npeaks+1)*5+2]="K"	+num2str((dsf_npeaks+1)*5+2)+" > 0"
				dsf_cnstr[(dsf_npeaks+1)*5+3]="K"	+num2str((dsf_npeaks+1)*5+3)+" > 0"
				dsf_cnstr[(dsf_npeaks+1)*5+4]=""
				dsf_npeaks+=1
			endif
		endif	
		dsf_coef[0]=dsf_npeaks	
	endif
End

Function FIT_addDoublet(): GraphMarquee 
	String ctrlname
	// data folder and stack
	String dfStr=SIS_getDataFolderName()
	SetDataFolder root:$(dfStr)
	//
	wave/T dsf_text=root:$(dfStr):dsf_text
	wave dsf_coef=root:$(dfStr):dsf_coef
	wave dsf_hold=root:$(dfStr):dsf_hold
	wave dsf_error=root:$(dfStr):dsf_error
	wave/T dsf_cnstr=root:$(dfStr):dsf_cnstr
	variable dsf_npeaks=dsf_coef[0]
	String tmpHoldStr=""
	variable c1,c2,c3,c4,c5,c6,c7,h1,h2,h3,h4,h5,h6,h7
	GetMarquee/Z/K left, bottom
	if (V_flag==1)
		if (dsf_npeaks==0)
			c1=0.1
			// Lorentzian width Gamma_L
			Prompt c1, "Lorentzian width (eV)"
			h1=2
			Prompt h1, "hold", popup "no;yes;"
			// Asymmetry alpha
			c2=0.1
			Prompt c2, "alpha (asymmetry)"
			h2=2
			Prompt h2, "hold", popup "no;yes;"
			//  Gaussian Width
			c3=0.5*abs(-V_left+V_right) 
			Prompt c3, "Gaussian width (eV)"
			h3=1
			Prompt h3, "hold", popup "no;yes;"
			// Intensity
			c4=(V_top-V_bottom)
			Prompt c4, "amplitude of 1st peak  (eV)"
			h4=1
			Prompt h4, "hold", popup "no;yes;"
			// Position
			c5=0.5*(V_left+V_right) 
			Prompt c5, "position of first peak (eV)"
			h5=1
			Prompt h5, "hold", popup "no;yes;"
			DoPrompt "Doniach Sunjic Doublet: 1st peak input", c1,h1,c2,h2,c3,h3,c4,h4,c5,h5
			if (V_flag==0)
				// occupation of second peak 
				c6=0.5
				Prompt c6, "2np peak occupation"
				h6=2
				Prompt h6, "hold", popup "no;yes;"
				// position of 2nd peak (realtive to first peak)
				c7=1.2
				Prompt c7, "realtive position of 2nd peak (eV)"
				h7=2
				Prompt h7, "hold", popup "no;yes;"
				DoPrompt "Doniach Sunjic Doublet: 2nd peak input", c6,h6,c7,h7
				if (V_flag==0)
					redimension/n=(5+(dsf_npeaks+1)*7) dsf_coef,dsf_hold,dsf_text,dsf_cnstr,dsf_error
					// stores into coef waves
					// guess
					dsf_coef[5]=c1
					dsf_coef[6]=c2 
					dsf_coef[7]=c3 
					dsf_coef[8]=c4 
					dsf_coef[9]=c5
					dsf_coef[10]=c6 
					dsf_coef[11]=c7
					// hold
					dsf_hold[5]=h1-1
					dsf_hold[6]=h2-1
					dsf_hold[7]=h3-1
					dsf_hold[8]=h4-1
					dsf_hold[9]=h5-1
					dsf_hold[10]=h6-1
					dsf_hold[11]=h7-1
					// text
					dsf_text[5]="Doublet Lorentzian width"
					dsf_text[6]="Doublet asymmetry"	
					dsf_text[7]="Doublet Gaussian width"	
					dsf_text[8]="1st peak Amplitude"	
					dsf_text[9]="1st peak Position"
					dsf_text[10]="2nd peak occupation"	
					dsf_text[11]="2nd peak Rel Position"			
					// constraints
					dsf_cnstr[5]="K5 > 0"	
					dsf_cnstr[6]="K6 > 0"		
					dsf_cnstr[7]="K7 > 0"	
					dsf_cnstr[8]="K8 > 0"
					dsf_cnstr[9]=""
					dsf_cnstr[10]="K10 > 0"
					dsf_cnstr[11]=""
					dsf_npeaks+=1
					dsf_coef[0]+=1
				endif
			endif
		else
			// Lorentzian width Gamma_L
			c1=dsf_coef[5+(dsf_npeaks-1)]  
			Prompt c1, "Lorentzian width"
			h1=dsf_hold[5+(dsf_npeaks-1)]+1
			Prompt h1, "hold", popup "no;yes;"
			// Asymmetry alpha
			c2=dsf_coef[5+(dsf_npeaks-1)+1]
			Prompt c2, "alpha (asimmetry)"
			h2=dsf_hold[5+(dsf_npeaks-1)+1]+1
			Prompt h2, "hold", popup "no;yes;"
			//  Gaussian Width
			c3=0.5*abs(-V_left+V_right) 
			Prompt c3, "Gaussian width (eV)"
			h3=dsf_hold[5+(dsf_npeaks-1)+2]+1
			Prompt h3, "hold", popup "no;yes;"
			// Intensity
			c4=(V_top-V_bottom)
			Prompt c4, "peak amplitude (eV)"
			h4=dsf_hold[5+(dsf_npeaks-1)+3]+1
			Prompt h4, "hold", popup "no;yes;"
			// Position
			c5=0.5*(V_left+V_right) 
			Prompt c5, "peak position (eV)"
			h5=dsf_hold[5+(dsf_npeaks-1)+4]+1
			Prompt h5, "hold", popup "no;yes;"
			DoPrompt "Doniach Sunjic: i-peak input", c1,h1,c2,h2,c3,h3,c4,h4,c5,h5
			if (V_flag==0)
				// occupation of second peak 
				c6=dsf_coef[5+(dsf_npeaks-1)+5]
				Prompt c6, "2np peak occupation"
				h6=2
				Prompt h6, "hold", popup "no;yes;"
				// position of 2nd peak (realtive to first peak)
				c7=dsf_coef[5+(dsf_npeaks-1)+6]
				Prompt c7, "realtive position of 2nd peak (eV)"
				h7=2
				Prompt h7, "hold", popup "no;yes;"
				DoPrompt "Doniach Sunjic Doublet: 2nd peak input", c6,h6,c7,h7
				if (V_flag==0)
					redimension/n=(5+(dsf_npeaks+1)*7) dsf_coef,dsf_hold,dsf_text,dsf_cnstr,dsf_error
					// guess
					dsf_coef[5+(dsf_npeaks)*7]=c1    
					dsf_coef[5+(dsf_npeaks)*7+1]=c2 
					dsf_coef[5+(dsf_npeaks)*7+2]=c3 
					dsf_coef[5+(dsf_npeaks)*7+3]=c4 
					dsf_coef[5+(dsf_npeaks)*7+4]=c5
					dsf_coef[5+(dsf_npeaks)*7+5]=c6
					dsf_coef[5+(dsf_npeaks)*7+6]=c7
					// hold
					dsf_hold[5+(dsf_npeaks)*7]=h1-1
					dsf_hold[5+(dsf_npeaks)*7+1]=h2-1 
					dsf_hold[5+(dsf_npeaks)*7+2]=h3-1 
					dsf_hold[5+(dsf_npeaks)*7+3]=h4-1 
					dsf_hold[5+(dsf_npeaks)*7+4]=h5-1
					dsf_hold[5+(dsf_npeaks)*7+5]=h6-1 
					dsf_hold[5+(dsf_npeaks)*7+6]=h7-1 
					// text
					dsf_text[5+(dsf_npeaks)*7]="Doublet Lorentzian width"
					dsf_text[5+(dsf_npeaks)*7+1]="Doublet asymmetry"	
					dsf_text[5+(dsf_npeaks)*7+2]="Doublet Gaussian width"	
					dsf_text[5+(dsf_npeaks)*7+3]="1st peak Amplitude"	
					dsf_text[5+(dsf_npeaks)*7+4]="1st peak Position"
					dsf_text[5+(dsf_npeaks)*7+5]="2nd peak occupation"	
					dsf_text[5+(dsf_npeaks)*7+6]="2nd peak Rel Position"			
					// constraints
					dsf_cnstr[5+(dsf_npeaks)*7]="K"+num2str(5+(dsf_npeaks)*7)+" > 0"
					dsf_cnstr[5+(dsf_npeaks)*7+1]="K"	+num2str(5+(dsf_npeaks)*7+1)+" > 0"	
					dsf_cnstr[5+(dsf_npeaks)*7+2]="K"	+num2str(5+(dsf_npeaks)*7+2)+" > 0"
					dsf_cnstr[5+(dsf_npeaks)*7+3]="K"	+num2str(5+(dsf_npeaks)*7+3)+" > 0"
					dsf_cnstr[5+(dsf_npeaks)*7+4]=""
					dsf_cnstr[5+(dsf_npeaks)*7+5]="K"	+num2str(5+(dsf_npeaks)*7+5)+" > 0"	
					dsf_cnstr[5+(dsf_npeaks)*7+6]=""
					// upload
					dsf_npeaks+=1
					dsf_coef[0]+=1
				endif
			endif
		endif	
		dsf_coef[0]=dsf_npeaks	
	endif
End

Function FIT_removePeak(): GraphMarquee
	String ctrlname
	// data folder and stack
	String dfStr=SIS_getDataFolderName()
	SetDataFolder root:$(dfStr)
	//
	wave/T dsf_text=root:$(dfStr):dsf_text
	wave dsf_coef=root:$(dfStr):dsf_coef
	wave dsf_hold=root:$(dfStr):dsf_hold
	wave/T dsf_cnstr=root:$(dfStr):dsf_cnstr
	wave dsf_error=root:$(dfStr):dsf_error
	variable npeak=dsf_coef[0]-1
	print npeak
	dsf_coef[0]=npeak
	variable n_coefs=(npeak+1)*5
	Redimension/n=(n_coefs) dsf_text,dsf_coef,dsf_hold,dsf_cnstr,dsf_error
End 

Function FIT_removeDoublet(): GraphMarquee
	String ctrlname
	// data folder and stack
	String dfStr=SIS_getDataFolderName()
	SetDataFolder root:$(dfStr)
	//
	wave/T dsf_text=root:$(dfStr):dsf_text
	wave dsf_coef=root:$(dfStr):dsf_coef
	wave dsf_hold=root:$(dfStr):dsf_hold
	wave dsf_error=root:$(dfStr):dsf_error
	wave/T dsf_cnstr=root:$(dfStr):dsf_cnstr
	variable npeak=dsf_coef[0]-1
	dsf_coef[0]=npeak
	variable n_coefs=5+(npeak)*7
	Redimension/n=(n_coefs) dsf_text,dsf_coef,dsf_hold,dsf_cnstr,dsf_error
End 

Function FIT_removeAllPeaks(): GraphMarquee
	// data folder and stack
	String dfStr=SIS_getDataFolderName()
	SetDataFolder root:$(dfStr)
	wave/T dsf_text=root:$(dfStr):dsf_text
	wave dsf_coef=root:$(dfStr):dsf_coef
	wave dsf_hold=root:$(dfStr):dsf_hold
	wave/T dsf_cnstr=root:$(dfStr):dsf_cnstr
	wave dsf_error=root:$(dfStr):dsf_error
	variable npeak=0
	dsf_coef[0]=npeak
	variable n_coefs=(npeak+1)*5
	Redimension/n=(n_coefs) dsf_text,dsf_coef,dsf_hold,dsf_cnstr,dsf_error
End 

Function FIT_multiPeakDS_old(w,x) : FitFunc
	Wave w
	Variable x
	// local vb
	Variable i=0
	Variable sumds=0
	Variable npeaks=w[0]
	Make/n=5/o wg
	// loop over n peaks
	sumds+=w[1]+w[2]*x+w[3]*x^2+w[4]*x^4
	for (i=1;i<=npeaks;i+=1)
			wg[0]=w[i*5]
			wg[1]=w[i*5+1]
			wg[2]=w[i*5+2]
			wg[3]=w[i*5+3]*10
			wg[4]=w[i*5+4]
// uncomment one of the two
			// using xop
// sumds+=dsgnnb(wg,x) 
			// using FIT function provided here
sumds+=FIT_DSsingle_KE(wg,x)
	endfor	
	killwaves wg
	return sumds
end

Function FIT_multiDoubletDS(w,x) : FitFunc
	Wave w
	Variable x
	String dfStr=SIS_getDataFolderName()
	wave shbkgy=root:$(dfStr):shbkgy
	wave shbkgx=root:$(dfStr):shbkgx
	NVAR backgroundtype=root:$(dfStr):backgroundtype
	// local vb
	Variable i=0
	Variable sumds=0
	Variable npeaks=w[0]
	Make/n=7/o wg
	// background
	sumds+=w[1]+w[2]*x+w[3]*x^2+w[4]*x^4
	// loop over n peaks
	for (i=1;i<=npeaks;i+=1)
			wg[0]=w[5+(i-1)*7]
			wg[1]=w[6+(i-1)*7]
			wg[2]=w[7+(i-1)*7]
			wg[3]=w[8+(i-1)*7]*10
			wg[4]=w[9+(i-1)*7]
			wg[5]=w[10+(i-1)*7]*w[8+(i-1)*7]*10
			wg[6]=w[11+(i-1)*7]
// uncomment one of the two
			// using xop
			// sumds+=dsgnmEasnb(wg,x)
			// using FIT function provided here 
			sumds+=FIT_DSdoublet_KE(wg,x)
	endfor	
	killwaves wg
	// shirley background
	variable shbkg=0
	if (backgroundtype==2)
		shbkg=interp(x, shbkgx, shbkgy)	
	endif
	return sumds+shbkg
end

Function FIT_multiPeakGauss(w,x) : FitFunc
	Wave w
	Variable x
	// local vb
	Variable i=0
	Variable sumgau=0
	Variable npeaks=w[0]
	Make/n=4/o wg
	// loop over n peaks
	sumgau+=w[1]+w[2]*x+w[3]*x^2+w[4]*x^3
	for (i=1;i<=npeaks;i+=1)
			wg[0]=0
			wg[1]=w[i*5+3]
			wg[2]=w[i*5+4]
			wg[3]=w[i*5+2]*sqrt(2)/2.3548
			sumgau+=Gauss1D(wg, x)
	endfor	
	killwaves wg
	return sumgau
end

Function FIT_multiPeakLor(w,x) : FitFunc
	Wave w
	Variable x
	// local vb
	Variable i=0
	Variable sumLor=0
	Variable npeaks=w[0]
	Make/n=4/o wg
	// loop over n peaks
	sumLor+=w[1]+w[2]*x+w[3]*x^2+w[4]*x^3
	for (i=1;i<=npeaks;i+=1)
			wg[0]=0
			wg[1]=w[i*5+3]
			wg[2]=w[i*5+4]
			wg[3]=(w[i*5]/2)^2
			sumLor+=abs(wg[1])/((x-wg[2])^2+wg[3])
	endfor	
	killwaves wg
	return sumLor
end

Function FIT_multiPeakDS(w,x) : FitFunc
	Wave w
	Variable x
	String dfStr=SIS_getDataFolderName()
	wave shbkgy=root:$(dfStr):shbkgy
	wave shbkgx=root:$(dfStr):shbkgx
	NVAR backgroundtype=root:$(dfStr):backgroundtype
	// local vb
	Variable i=0
	Variable sumds=0
	Variable npeaks=w[0]
	Make/n=5/o wg
	// loop over n peaks
	sumds+=w[1]+w[2]*x+w[3]*x^2+w[4]*x^4
	for (i=1;i<=npeaks;i+=1)
		if  ((w[i*5]!=0) && (w[i*5+2]!=0))
			wg[0]=w[i*5]
			wg[1]=w[i*5+1]
			wg[2]=w[i*5+2]
			wg[3]=w[i*5+3]*10
			wg[4]=w[i*5+4]
// uncomment one of the two
			// using xop
// sumds+=dsgnnb(wg,x) 
			// using FIT function provided here
sumds+=FIT_DSsingle_KE(wg,x)
		else
			if (w[i*5+2]==0) // means purely lorentzian
				wg[0]=0
				wg[1]=w[i*5+3]
				wg[2]=w[i*5+4]
				wg[3]=(w[i*5]/2)^2
				sumds+=abs(wg[1])/((x-wg[2])^2+wg[3])
			endif
			if (w[i*5]==0) // means purely gaussian
				wg[0]=0
				wg[1]=w[i*5+3]
				wg[2]=w[i*5+4]
				wg[3]=w[i*5+2]*sqrt(2)/2.3548
				sumds+=Gauss1D(wg, x)
			endif
		endif
	endfor
	killwaves wg
	// shirley background
	variable shbkg=0
	//if (backgroundtype==2)
	//	shbkg=interp(x, shbkgx, shbkgy)	
	//endif
	return sumds+shbkg
end

Function FIT_Lor1D(w,x):FitFunc
	Wave w
	Variable x
	return w[0]+abs(w[1])/((x-w[2])^2+(w[3]/2)^2)
end

Function FIT_PolyBackground(w,x) : FitFunc
	Wave w
	Variable x
	// local vb
	Variable i=0
	Variable sumpd=0
	Make/n=5/o wg
	sumpd+=w[1]+w[2]*x+w[3]*x^2+w[4]*x^3
	return sumpd
end

Function FIT_stepFunction(w,x) : FitFunc
	Wave w
	Variable x
	variable npeaks=w[0]
	variable x0=w[5]
	variable sigma=abs(w[6])/(2*sqrt(ln(2))*sqrt(2)) //sigma=width/2.35
	variable amp=w[7]
	variable nsteps = 12
	variable integral = 0
	variable howfar=2.4
	variable start = -1*howfar*sigma
	variable stop = howfar*sigma
	variable delta = 2*howfar/nsteps
	variable yy=start
	Variable normG = 1/sqrt(2*Pi*sigma^2)
	variable bkg
	do
      		if ((x+yy)<x0)
      			bkg=w[1]+w[2]*(x+yy-x0)+w[3]*(x+yy-x0)^2+w[4]*(x+yy-x0)^3
      		else
      			bkg=w[1]
      		endif
      		integral+=normG*exp(-(yy)^2/(2*sigma^2))*(w[1]+amp/(1+exp((x+yy-x0)/0.01)))*delta                               
		yy+=delta 	
	while (yy <= stop)
	return integral
end

Function FIT_sigmoid(w,x) : FitFunc
	Wave w
	variable x
	variable x0=w[5]
	variable npeaks=w[0]
	variable bkg
	variable xhalf=w[6]
	//if (x<x0-xhalf)
      	//	bkg=w[1] +w[2]*(x-(x0-xhalf))+w[3]*(x-(x0-xhalf))^2+w[4]*(x-(x0-xhalf))^3
      	//else
      	//	bkg=w[1]
      	//endif
	//return bkg+w[7]/(1+exp((x-x0)/(0.5*sqrt(2)*w[6]/2.35)))  //fwhm=2.35*sigma=2.35*gwidth/sqrt(2)
	if (x<x0)
      		bkg=w[1]+w[2]*(x-x0)+w[3]*(x-x0)^2+w[4]*(x-x0)^3
      	else
      		bkg=w[1]
      	endif
	return bkg+w[7]/(1+exp((x-x0)/(0.5*sqrt(2)*w[6]/2.35)))  //fwhm=2.35*sigma=2.35*gwidth/sqrt(2)
end

Function FIT_makeHoldStr()
	// data folder
	String dfStr=SIS_getDataFolderName()
	SetDataFolder root:$(dfStr)
	// globals
	wave/T dsf_text=root:$(dfStr):dsf_text
	wave dsf_coef=root:$(dfStr):dsf_coef
	wave dsf_hold=root:$(dfStr):dsf_hold
	wave/T dsf_cnstr=root:$(dfStr):dsf_cnstr
	SVAR holdStr=root:$(dfStr):holdStr
	// convert array to string 
	variable holdStrLength=numpnts(dsf_coef)
	variable i=0
	holdStr=""
	for(i=0;i<holdStrLength;i+=1) 
		holdStr+=num2str(dsf_hold[i])
	endfor
	redimension/n=(holdStrLength) dsf_hold
End

Function/S FIT_makeCnstrStr(tmpHoldStr)
	String tmpHoldStr
	// data folder and stack
	String dfStr=SIS_getDataFolderName()
	SetDataFolder root:$(dfStr)
	// globals
	wave/T dsf_text=root:$(dfStr):dsf_text
	wave dsf_coef=root:$(dfStr):dsf_coef
	wave dsf_hold=root:$(dfStr):dsf_hold
	wave/T dsf_cnstr=root:$(dfStr):dsf_cnstr
	SVAR holdStr=root:$(dfStr):holdStr
	make/n=0/o/T dsf_fit_cnstr
	wave/T dsf_fit_cnstr=root:$(dfStr):dsf_fit_cnstr
	// convert constraint array to string only for 
	variable i=0
	variable j=0
	variable iend=numpnts(dsf_coef)  
	for(i=0;i<iend;i+=1) 
		if ((strsearch(dsf_cnstr[i],"K",0)==0) && (stringmatch(tmpHoldStr[i],"0")==1))
			InsertPoints j,1, dsf_fit_cnstr
			dsf_fit_cnstr[j]=dsf_cnstr[i]
			j+=1
		endif
	endfor
	return dsf_fit_cnstr 
End

Function FIT_flattenAll()
       // data folder and stack
	String dfStr=SIS_getDataFolderName()
	SetDataFolder root:$(dfStr)
	// xwave
	String winStr=SIS_getWinName()
	// globals
	wave/T dsf_text=root:$(dfStr):dsf_text
	wave dsf_coef=root:$(dfStr):dsf_coef
	wave dsf_hold=root:$(dfStr):dsf_hold
	wave/T dsf_cnstr=root:$(dfStr):dsf_cnstr
	//wave/T dsf_fit_cnstr=root:$(dfStr):dsf_fit_cnstr
	SVAR holdStr=root:$(dfStr):holdStr
	NVAR fittype=root:$(dfStr):fittype
	string ywaveStr
	Prompt ywaveStr, "enter Y wave",popup,WaveList("!*_P",";","WIN:")
	string xwaveStr
	Prompt xwaveStr, "enter X wave",popup,WaveList("*_P",";","WIN:")
	doPrompt "input fit"ywaveStr,xwaveStr
	if (V_flag==0)
		variable npeaks=dsf_coef[0]
		string peakstr
		string bkgStr=ywaveStr+"_BKG"
		duplicate/o $ywaveStr $bkgStr
		wave bk=$bkgStr
		wave wx=$xwaveStr
		duplicate/o w_dsf_coef w_dsf_tmp
		w_dsf_tmp[5,numpnts(w_dsf_tmp)-1]=0
		bk[]=FIT_PolyBackground(w_dsf_tmp,wx[p])
		variable i
		for(i=1;i<=npeaks;i+=1)
			// wave of i-th component 
			peakStr=ywaveStr+"_"+num2str(i)
			wave wp=$peakStr
			wp-=bk
		endfor
		wave wy=$ywaveStr
		wy-=bk
		//
		duplicate/o w_dsf_coef w_dsf_tmp
		w_dsf_tmp[1,4]=0
		string fitStr="fit_"+ywaveStr
		wave wf=$fitStr
		wf[]=FIT_multiPeakDS(w_dsf_tmp,x)
	endif
end

Function FIT_fitProfile()
	// data folder and stack
	String dfStr=SIS_getDataFolderName()
	SetDataFolder root:$(dfStr)
	// xwave
	String winStr=SIS_getWinName()
	String xwaveStr=winStr+"_P"
	// globals
	wave/T dsf_text=root:$(dfStr):dsf_text
	wave dsf_coef=root:$(dfStr):dsf_coef
	wave dsf_hold=root:$(dfStr):dsf_hold
	wave/T dsf_cnstr=root:$(dfStr):dsf_cnstr
	//
	wave shbkgy=root:$(dfStr):shbkgy
	wave shbkgx=root:$(dfStr):shbkgx
	//wave/T dsf_fit_cnstr=root:$(dfStr):dsf_fit_cnstr
	SVAR holdStr=root:$(dfStr):holdStr
	NVAR fittype=root:$(dfStr):fittype
	NVAR backgroundtype=root:$(dfStr):backgroundtype
	// pause for user input
	SIS_PauseForCursors(winStr,"Adjust crs A and B")	
	// cursors
	variable fh,tt
	if (strlen(CsrInfo(A))==0)
		fh=0
	else
		fh=pcsr(A)
	endif
	if (strlen(CsrInfo(B))==0)
		tt=dimSize($xwaveStr,0)-1
	else
		tt=pcsr(B)
	endif
	// input 
	FIT_makeHoldStr()
	String ywaveStr
	Prompt ywaveStr, "enter Y wave",popup,WaveList("!*_P",";","WIN:")
	Prompt xwaveStr, "enter X wave",popup,WaveList("*_P",";","WIN:")
	String tmpHoldStr=holdStr
	Prompt tmpHoldStr, "hold"
	Variable fromhere=fh
	Prompt fromhere, "start at point"
	Variable tothere=tt
	Prompt tothere, "end at point"	
	Variable mode=fittype
	Prompt mode, "Function type", popup "Doniach-Sunjic;Gaussian;Lorentzian;Step;Sigmoid;"
	Variable background=backgroundtype
	Prompt background, "Function type", popup "linear;Shirley;"
	doPrompt "input fit", xwaveStr,ywaveStr,tmpHoldStr,fromHere,tothere,mode,background
	// if ok
	if (V_flag==0)  
		fittype=mode
		backgroundtype=background
		duplicate/o dsf_coef w_dsf_coef
		FIT_makeCnstrStr(tmpHoldStr)
		wave/T dsf_fit_cnstr=root:$(dfStr):dsf_fit_cnstr
		// debug
		//print tmpHoldStr
		//print dsf_fit_cnstr
		//print w_dsf_coef
		//  shirley
		duplicate/o $ywaveStr shbkgy
		duplicate/o $xwaveStr shbkgx
		FIT_calculateShirleyBackground("shbkgy","shbkgx",5)
		// do the fit
		if (mode == 1) // DS
			if (strsearch(dsf_text[6],"Doublet",0)==-1)
				print "Fit type: Doniach-Sunjic"
				FuncFit/H=(tmpHoldStr)/G/NTHR=0 FIT_multiPeakDS w_dsf_coef $ywaveStr[fromhere,tothere] /X=$xwaveStr /D /R /C=dsf_fit_cnstr
			else
				print "Fit type: Doniach-Sunjic Doublet"
				FuncFit/H=(tmpHoldStr)/G/NTHR=0 FIT_multiDoubletDS w_dsf_coef $ywaveStr[fromhere,tothere] /X=$xwaveStr /D /R /C=dsf_fit_cnstr
			endif
		endif
		if (mode == 2) // Gauss
			if (strsearch(dsf_text[6],"Doublet",0)==-1)
				print "Fit type: Gaussian"
				FuncFit/H=(tmpHoldStr)/G/NTHR=0 FIT_multiPeakGauss w_dsf_coef $ywaveStr[fromhere,tothere] /X=$xwaveStr /D /R /C=dsf_fit_cnstr
			endif
		endif
		if (mode == 3) //Lor
			if (strsearch(dsf_text[6],"Doublet",0)==-1)
				print "Fit type: Lorentzian"
				FuncFit/H=(tmpHoldStr)/G/NTHR=0  FIT_multiPeakLor w_dsf_coef $ywaveStr[fromhere,tothere] /X=$xwaveStr /D /R /C=dsf_fit_cnstr
			endif
		endif
		if (mode == 4) //Step
			print "Fit type: Step function (Fermi edge)"
			FuncFit/H=(tmpHoldStr)/G/NTHR=0 FIT_stepFunction w_dsf_coef $ywaveStr[fromhere,tothere] /X=$xwaveStr /D /R /C=dsf_fit_cnstr
		endif
		if (mode == 5) //Sigmoid
			print "Fit type: Sigmoid"
			FuncFit/H=(tmpHoldStr)/G/NTHR=0  FIT_sigmoid w_dsf_coef $ywaveStr[fromhere,tothere] /X=$xwaveStr /D /R /C=dsf_fit_cnstr
		endif
	       // interactive
		variable updateCoef=1
		Prompt updateCoef, "Update startup coefficients", popup "yes; no;"
		doPrompt "Update fit cofficient", updateCoef
		if ((V_flag == 0) && (updateCoef==1))
			// update coefficients
			duplicate/o w_dsf_coef dsf_coef
			duplicate/o W_dsf_coef dsf_error
			wave W_sigma
			dsf_error=W_sigma
			// plot components
			if (mode < 4)
				FIT_PlotComponents(dfstr,winStr,ywaveStr,xwaveStr,mode)
			endif
		endif
	endif
end

Function FIT_PlotComponents(dfstr,winStr,ywaveStr,xwaveStr,mode)
	String dfStr,ywaveStr,xwaveStr,winStr
	variable mode
	// globals
	wave/T dsf_text=root:$(dfStr):dsf_text
	wave dsf_coef=root:$(dfStr):dsf_coef
	wave w_dsf_coef=root:$(dfStr):w_dsf_coef
	wave dsf_hold=root:$(dfStr):dsf_hold
	wave/T dsf_cnstr=root:$(dfStr):dsf_cnstr
	wave/T dsf_fit_cnstr=root:$(dfStr):dsf_fit_cnstr
	NVAR backgroundtype=root:$(dfStr):backgroundtype
	wave shbkgy=root:$(dfStr):shbkgy
	wave shbkgx=root:$(dfStr):shbkgx
	// sho components
	variable npeaks=w_dsf_coef[0]
	string peakStr
	string graphStr=SIS_getWinName()
	variable i,j
	variable integral
	string messageStr=""
	string tagStr=""
	string lorStr=""
	string gauStr=""
	string posStr=""
	string asymStr=""
	j=1
	print npeaks
	for(i=1;i<=npeaks;i+=1)
		// wave of i-th component 
		peakStr="fit_"+ywaveStr+"_"+num2str(i)
		duplicate/o $ywaveStr $peakStr
		wave wpeak=$peakStr
		wave wx=$xwaveStr
		// 1: compute peak using DS fitting function
		// this is the doublet case
		if (strsearch(dsf_text[6],"Doublet",0)==0)
			make/n=7/o w_dsf_tmp
			 w_dsf_tmp[0]=w_dsf_coef[5+(i-1)*7]
			 w_dsf_tmp[1]=w_dsf_coef[6+(i-1)*7]
			 w_dsf_tmp[2]=w_dsf_coef[7+(i-1)*7]
			 w_dsf_tmp[3]=w_dsf_coef[8+(i-1)*7]*10
			 w_dsf_tmp[4]=w_dsf_coef[9+(i-1)*7]
			 w_dsf_tmp[5]=w_dsf_coef[10+(i-1)*7]*w_dsf_coef[8+(i-1)*7]*10
			 w_dsf_tmp[6]=w_dsf_coef[11+(i-1)*7]
			 posStr=num2Str(w_dsf_tmp[4])
			 lorStr=num2Str(w_dsf_tmp[0])
			 asymStr=num2Str(w_dsf_tmp[1])
			 gauStr=num2Str(w_dsf_tmp[2])
			if (mode==1)
				// uncomment one of the two
				// using xop				
// wpeak[]=dsgnmEasnb(w_dsf_tmp,wx[p])
				// using function provided here
				wpeak[]=FIT_DSdoublet_KE(w_dsf_tmp,wx[p])
			endif
		else
			//make/n=5/o w_dsf_tmp
			if (mode==1)
				make/n=(10)/o w_dsf_tmp
			 	w_dsf_tmp[0]=1
			 	w_dsf_tmp[1]=0
			 	w_dsf_tmp[2]=0
			 	w_dsf_tmp[3]=0
			 	w_dsf_tmp[4]=0
			 	w_dsf_tmp[5]=w_dsf_coef[5+(i-1)*5]
				w_dsf_tmp[6]=w_dsf_coef[6+(i-1)*5]
				w_dsf_tmp[7]=w_dsf_coef[7+(i-1)*5]
				w_dsf_tmp[8]=w_dsf_coef[8+(i-1)*5]
			 	w_dsf_tmp[9]=w_dsf_coef[9+(i-1)*5]
			 	posStr=num2Str(w_dsf_tmp[9])
			 	lorStr=num2Str(w_dsf_tmp[5])
			 	asymStr=num2Str(w_dsf_tmp[6])
			 	gauStr=num2Str(w_dsf_tmp[7])
			 	wpeak[]=FIT_multiPeakDS(w_dsf_tmp,wx[p])
				//w_dsf_tmp[0]=w_dsf_coef[5+(i-1)*5]
				//w_dsf_tmp[1]=w_dsf_coef[6+(i-1)*5]
			 	//w_dsf_tmp[2]=w_dsf_coef[7+(i-1)*5]
				//w_dsf_tmp[3]=w_dsf_coef[8+(i-1)*5]*10
			 	//w_dsf_tmp[4]=w_dsf_coef[9+(i-1)*5]
				// uncomment one of the two
				// using xop
// wpeak[]=dsgnnb(w_dsf_tmp,wx[p])
				// using function provided here
				// wpeak[]=FIT_DSsingle_KE(w_dsf_tmp,wx[p])
			endif
			if (mode == 2)
				make/n=(10)/o w_dsf_tmp
			 	w_dsf_tmp[0]=1
			 	w_dsf_tmp[1]=0
			 	w_dsf_tmp[2]=0
			 	w_dsf_tmp[3]=0
			 	w_dsf_tmp[4]=0
			 	w_dsf_tmp[5]=w_dsf_coef[5+(i-1)*5]
				w_dsf_tmp[6]=w_dsf_coef[6+(i-1)*5]
				w_dsf_tmp[7]=w_dsf_coef[7+(i-1)*5]
				w_dsf_tmp[8]=w_dsf_coef[8+(i-1)*5]
			 	w_dsf_tmp[9]=w_dsf_coef[9+(i-1)*5]
			 	posStr=num2Str(w_dsf_tmp[9])
			 	lorStr=num2Str(w_dsf_tmp[5])
			 	asymStr=num2Str(w_dsf_tmp[6])
			 	gauStr=num2Str(w_dsf_tmp[7])
				wpeak[]=FIT_multiPeakGauss(w_dsf_tmp,wx[p])
			endif
			if (mode == 3)
				make/n=(10)/o w_dsf_tmp
			 	w_dsf_tmp[0]=1
			 	w_dsf_tmp[1]=0
			 	w_dsf_tmp[2]=0
			 	w_dsf_tmp[3]=0
			 	w_dsf_tmp[4]=0
				w_dsf_tmp[5]=0
				w_dsf_tmp[6]=w_dsf_coef[6+(i-1)*5]
				w_dsf_tmp[7]=w_dsf_coef[7+(i-1)*5]
				w_dsf_tmp[8]=w_dsf_coef[i*5]
			 	w_dsf_tmp[9]=w_dsf_coef[9+(i-1)*5]
			 	posStr=num2Str(w_dsf_tmp[9])
			 	lorStr=num2Str(w_dsf_tmp[5])
			 	asymStr=num2Str(w_dsf_tmp[6])
			 	gauStr=num2Str(w_dsf_tmp[7])
				wpeak[]=FIT_Lor1D(w_dsf_tmp,wx[p])
			endif
		endif
		// 2: calculate peak integral 
		messageStr=""
		if (mode < 4)
			Redimension/D/n=(numpnts(wpeak)) wpeak,wx
			Integrate/METH=1 wpeak /X=wx /D=wintegral
			integral=wintegral[numpnts(wintegral)-1]
			messageStr="peak no."+num2str(i)+" at "+posStr+" eV\r"
			messageStr+="Lorentzian width = "+lorStr+" eV\r"
			messageStr+="Gaussian width = "+gauStr+" eV\r"
			messageStr+="asymmetry alpha = "+asymStr+" \r"
			messageStr+="peak integral = "+num2Str(integral)
		endif
		// 3: calculate background
		duplicate/o w_dsf_coef w_dsf_tmp
		w_dsf_tmp[5,numpnts(w_dsf_tmp)-1]=0
		wpeak[]+=FIT_PolyBackground(w_dsf_tmp,wx[p])
		//if (backgroundtype == 2)
		//      wpeak+=shbkgy
		//endif
		//append, if needed
		SIS_graphToFront(graphStr,peakStr,xwaveStr)
		// append tag
		tagStr="tag_"+ywaveStr+num2Str(j)
		WaveStats/Q wpeak
		Tag/C/N=$(tagStr)/L=2/A=MB/TL=0 $peakStr, V_maxloc, messageStr
		doUpdate
		j+=1
	endfor
	//killwaves w_dsf_tmp, wintegral	
end

Function FIT_Calculate_Fit_Integrals()
       // data folder and stack
	String dfStr=SIS_getDataFolderName()
	SetDataFolder root:$(dfStr)
	// globals
	wave/T dsf_text=root:$(dfStr):dsf_text
	wave dsf_coef=root:$(dfStr):dsf_coef
	wave w_dsf_coef=root:$(dfStr):w_dsf_coef
	wave dsf_hold=root:$(dfStr):dsf_hold
	wave/T dsf_cnstr=root:$(dfStr):dsf_cnstr
	wave/T dsf_fit_cnstr=root:$(dfStr):dsf_fit_cnstr
	// variables
	String winStr=SIS_getWinName()
	String xwaveStr=winStr+"_P"
	String ywaveStr
	NVAR fittype=root:$(dfStr):fittype
	// input
	Prompt ywaveStr, "enter wave",popup,WaveList("!*_P",";","WIN:")
	Variable mode=fittype
	Prompt mode, "Function type", popup "Doniach-Sunjic;Gaussian;Lorentzian;Step;Sigmoid;"
	Variable range=25
	Prompt range, "range (eV)"
	doPrompt "input fit", ywaveStr,mode,range
	// if ok
	if (V_flag==0)
	// sho components
	variable npeaks=dsf_coef[0]
	string peakStr
	string graphStr=SIS_getWinName()
	variable i,j
	variable integral
	string messageStr=""
	string tagStr=""
	j=1
	variable nint=1000
	make/N=(nint)/O tmpx=NaN,tmpy=NaN,wintegral=NaN
	tmpx=0
	tmpy=0
	for(i=1;i<=npeaks;i+=1)
		// 1: compute peak using fitting function
		if (strsearch(dsf_text[6],"Doublet",0)==0)
			make/n=7/o w_dsf_tmp
			 w_dsf_tmp[0]=w_dsf_coef[5+(i-1)*7]
			 w_dsf_tmp[1]=w_dsf_coef[6+(i-1)*7]
			 w_dsf_tmp[2]=w_dsf_coef[7+(i-1)*7]
			 w_dsf_tmp[3]=w_dsf_coef[8+(i-1)*7]*10
			 w_dsf_tmp[4]=w_dsf_coef[9+(i-1)*7]
			 w_dsf_tmp[5]=w_dsf_coef[10+(i-1)*7]*w_dsf_coef[8+(i-1)*7]*10
			 w_dsf_tmp[6]=w_dsf_coef[11+(i-1)*7]
			tmpx[]=w_dsf_tmp[4]-range+p*(2*range)/nint  
			if (mode==1)
// uncomment one of the two
				// using xop				
// wpeak[]=dsgnmEasnb(w_dsf_tmp,wx[p])
				// using function provided here
tmpy[]=FIT_DSdoublet_KE(w_dsf_tmp,tmpx[p])
			endif
		else
			make/n=5/o w_dsf_tmp
			if (mode==1)
				w_dsf_tmp[0]=w_dsf_coef[5+(i-1)*5]
				w_dsf_tmp[1]=w_dsf_coef[6+(i-1)*5]
			 	w_dsf_tmp[2]=w_dsf_coef[7+(i-1)*5]
				w_dsf_tmp[3]=w_dsf_coef[8+(i-1)*5]*10
			 	w_dsf_tmp[4]=w_dsf_coef[9+(i-1)*5]
			 	tmpx[]=w_dsf_tmp[4]-range+p*(2*range)/nint
// uncomment one of the two
				// using xop
// wpeak[]=dsgnnb(w_dsf_tmp,wx[p])
				// using function provided here
tmpy[]=FIT_DSsingle_KE(w_dsf_tmp,tmpx[p])
			endif
			if (mode == 2)
			 	w_dsf_tmp[0]=0
				w_dsf_tmp[1]=w_dsf_coef[6+(i-1)*5]
				w_dsf_tmp[2]=w_dsf_coef[7+(i-1)*5]
				w_dsf_tmp[3]=w_dsf_coef[8+(i-1)*5]
			 	w_dsf_tmp[4]=w_dsf_coef[9+(i-1)*5]
				w_dsf_tmp[3]/=10
				tmpx[]=w_dsf_tmp[4]-range+p*(2*range)/nint
				tmpy[]=FIT_multiPeakGauss(w_dsf_tmp,tmpx[p])
			endif
			if (mode == 3)
				w_dsf_tmp[0]=0
				w_dsf_tmp[1]=w_dsf_coef[6+(i-1)*5]
				w_dsf_tmp[2]=w_dsf_coef[7+(i-1)*5]
				w_dsf_tmp[3]=w_dsf_coef[i*5]
			 	w_dsf_tmp[4]=w_dsf_coef[9+(i-1)*5]
			 	tmpx[]=w_dsf_tmp[4]-range+p*(2*range)/nint
				tmpy[]=FIT_Lor1D(w_dsf_tmp,tmpx[p])
			endif
		endif
		// 2: calculate peak integral 
		messageStr=""
		if (mode < 3)
			Integrate/METH=1 tmpy /X=tmpx /D=wintegral
			integral=wintegral[numpnts(wintegral)-1]
			print "range around peak maximum: +/-",range," eV"
			messageStr="peak no."+num2str(i)+" at "+num2Str(round(w_dsf_tmp[4]))+" eV, "+"integral = "+num2Str(integral)
			print messageStr
		endif
		j+=1
	endfor
	killwaves w_dsf_tmp, wintegral, tmpx, tmpy	
	endif
end

function FIT_normalizeSpectrum()
	// data folder and stack
	String dfStr=SIS_getDataFolderName()
	SetDataFolder root:$(dfStr)
	// xwave
	String winStr=SIS_getWinName()
	// pause for user input
	SIS_PauseForCursors(winStr,"Adjust crs A and B")	
	variable p1=min(pcsr(A),pcsr(B))
	variable p2=max(pcsr(A),pcsr(B))
	variable i=0
	// y wave
	String ywaveStr
	Prompt ywaveStr, "enter wave",popup,WaveList("!*_P",";","WIN:")
	doPrompt "Shirely Background", ywaveStr
	if (V_Flag==0) 
		variable mvm=0
		variable mv=mean($ywaveStr,p1,p2)
		print mv
		wave w=$ywaveStr
		w/=mv
	endif
end

Function FIT_removeShirleyBackground()
  	// based on D.A. Shirley, phys. Rev. B 5(12), 4709-4714 (1972).
  	// data folder and stack
	String dfStr=SIS_getDataFolderName()
	SetDataFolder root:$(dfStr)
	// xwave
	String winStr=SIS_getWinName()
	// pause for user input
	SIS_PauseForCursors(winStr,"Adjust crs A and B")	
	// y wave
	String ywaveStr
	Prompt ywaveStr, "enter wave",popup,WaveList("!*_P",";","WIN:")
	String xwaveStr
	Prompt xwaveStr, "enter wave",popup,WaveList("*_P",";","WIN:")
	variable npntsbkg=2
	Prompt npntsbkg, "number of bkground points"
	doPrompt "Shirely Background", ywaveStr, xwaveStr, npntsbkg
	if (V_Flag==0) 
		// names
		String shirleyStr="SH"+ywaveStr[0,strlen(ywaveStr)]
		String shirleyxStr="SH"+dfStr+"_P"
		String bkgStr="BK"+ywaveStr[0,strlen(ywaveStr)]
		// get cursors
		variable pxa=min(pcsr(A),pcsr(B))
		variable pxb=max(pcsr(A),pcsr(B))
		//pxb=min(pxb,numpnts($ywaveStr))
		variable m=abs(pxb-pxa)
		// create waves
		make/n=(m)/o $shirleyStr,$shirleyxStr, $bkgStr
		wave sw=$shirleyStr
		wave sx=$shirleyxStr
		wave xw=$xwaveStr
		wave yw=$ywaveStr
		wave bw=$bkgStr
		// shirely X wave
		sw[]=yw[p+pxa]
       	sx[]=xw[p+pxa]
       	variable n1=xw[pxa]								
		variable n2=xw[pxb]	
		print pxa,pxb	
		// shirley Y wave 
		bw[] = FIT_Shirley(xw[pxa+p],xw[pxb-1],xw,yw) // numerator of (3) at page 4711)
		variable denom=bw[0]
		bw/=denom
		// now we scale to the values at pxa and pxb
		variable v1,v2,height
       	wavestats/Q/R=(pxa,pxa+npntsbkg) yw
       	v1=V_avg
       	wavestats/Q/R=(pxb-npntsbkg,pxb) yw
       	v2=V_avg
       	height=min(v1,v2) 
       	variable scalingfactor=abs(v1-v2) 
       	bw*=scalingfactor
       	bw+=height
       	sw[] = yw[p+pxa]-bw[p]
       	String resultStr=""
       	if (n1<n2)
       		resultStr=" the peak area is: "+num2Str(AreaXY(sx,sw))+" \r the bkg avg value is: "+num2Str(v1) 
       	else
       		resultStr=" the peak area is: "+num2Str(AreaXY(sx,sw))+" \r the bkg avg value is: "+num2Str(v1) 
       	endif
       	print resultStr
       	String graphStr="SH"+dfStr
       	SIS_graphToFront(graphStr,shirleyStr,shirleyxStr)
       	SIS_graphToFront(graphStr,shirleyStr,shirleyxStr)
       	SIS_graphToFront(graphStr,bkgStr,shirleyxStr)
       	Label bottom "\\F'Arial'\\Z18x axis label (Arb. Units)"
       	Label left "\\F'Arial'\\Z18Intensity (Arb. Units.)"
		ModifyGraph fSize=12
		ModifyGraph width=226.772,height={Aspect,1}
		ModifyGraph log=0
		SetAxis/A bottom
		TextBox/C/N=text0/A=RT resultStr
	  endif
  end 
  
  Function FIT_Shirley(x1,x2,wx,wy)
	variable x1,x2
	wave wx, wy
	return areaXY(wx, wy, x1, x2 )
End 
  
Function FIT_removeShirleyFromProfile(ywaveStr,xwaveStr,npntsbkg)
	String ywaveStr
	String xwaveStr
	variable npntsbkg
	wave yw=$ywaveStr
	wave xw=$xwaveStr
 	variable pxa=0
	variable pxb=numpnts($ywaveStr)
	variable m=abs(pxb-pxa)
	duplicate/o $ywaveStr bw
	// shirely X wave
      	variable x=xw[pxa]								
	variable n=xw[pxb]		
	// shirley Y wave 
	variable v1,v2
      	v1=mean(yw,pxa,pxa+npntsbkg) 						
     	v2=mean(yw,pxb-npntsbkg,pxb) 
     	bw = FIT_Shirley(xw[p],xw[m-1],xw,yw)
     	variable scalingfactor=(v1-v2)/bw[0]
       bw*=scalingfactor
       bw+=v2
       yw[] = yw[p]-bw[p]
       duplicate/o yw W_ExtractedCol
End

Function FIT_calculateShirleyBackground(ywaveStr,xwaveStr,npntsbkg)
	String ywaveStr
	String xwaveStr
	variable npntsbkg
	wave yw=$ywaveStr
	wave xw=$xwaveStr
 	variable pxa=0
	variable pxb=numpnts($ywaveStr)
	variable m=abs(pxb-pxa)
	duplicate/o $ywaveStr bw
	// shirley back wave 
	bw[] = FIT_Shirley(xw[pxa+p],xw[pxb],xw,yw) // numerator of (3) at page 4711)
	bw[m]=0
	variable denom=bw[0]
	bw/=denom
	// now we scale to the values at pxa and pxb
	variable v1,v2,height
       wavestats/Q/R=(pxa,pxa+npntsbkg) yw
       v1=V_avg
       wavestats/Q/R=(pxb-npntsbkg,pxb) yw
       v2=V_avg
       height=min(v1,v2) 
       variable scalingfactor=abs(v1-v2) 
       bw*=scalingfactor
       bw+=height
       yw[] = bw[p]
End
 
Function FIT_fitSeries()
	// data folder and stack
	String dfStr=SIS_getDataFolderName()
	SetDataFolder root:$(dfStr)
	// waves
	String wfStr=SIS_getImageStackName()
	String winStr=SIS_getWinName()
	String xStr=winStr+"_P"
	// globals
	wave/T dsf_text=root:$(dfStr):dsf_text
	wave dsf_coef=root:$(dfStr):dsf_coef
	wave dsf_hold=root:$(dfStr):dsf_hold
	wave/T dsf_cnstr=root:$(dfStr):dsf_cnstr
	wave/T dsf_fit_cnstr=root:$(dfStr):dsf_fit_cnstr
	SVAR holdStr=root:$(dfStr):holdStr
	NVAR fittype=root:$(dfStr):fittype
	FIT_makeHoldStr()
	// pause for user input
	SIS_PauseForCursors(winStr,"Adjust crs A and B")	
	// cursors
	variable ff,tf,fh,tt
	if (strlen(CsrInfo(A))==0)
		ff=0
		fh=0
	else
		ff=qcsr(A)
		fh=pcsr(A)
	endif
	if (strlen(CsrInfo(B))==0)
		tf=dimSize($wfStr,1)-1
		tt=dimSize($wfStr,1)-1
	else
		tf=qcsr(B)
		tt=pcsr(B)
	endif
	// input 
	Variable fromFrame=ff
	Prompt fromFrame, "From Frame"
	Variable toFrame=tf
	Prompt toFrame, "to Frame"
	Variable fromhere=fh
	Prompt fromhere, "start at point"
	Variable tothere=tt
	Prompt tothere, "end at point"
	String tmpHoldStr=holdStr
	Prompt tmpHoldStr, "hold"
	Variable mode=fittype
	Prompt mode, "Function type", popup "Doniach-Sunjic;Gaussian;Lorentzian;Step;Sigmoid;"
	Variable shirley=1
	Prompt shirley, "Shirley background", popup "keep;remove;"
	Variable npntsbkg=6
	Prompt npntsbkg, "n points for Shirley background"
	doPrompt "input fit series WF ", fromFrame,toFrame,fromhere,tothere,tmpHoldStr,mode,shirley,npntsbkg
	// if ok
	if (V_flag==0)  
		fittype=mode
		// prepare for fit and create the results wave
		FIT_makeCnstrStr(tmpHoldStr)
		duplicate/o dsf_coef w_dsf_coef 
		variable nq=dimsize($wfStr,1)
		variable n_of_coefficients=(dsf_coef[0]+1)*5
		make/n=(nq,n_of_coefficients)/o fit_results=NaN
		make/n=(nq,n_of_coefficients)/o fit_errors=NaN
		// new graph
		variable i=fromFrame
		imageTransform/G=(i) getCol $wfStr
		duplicate/o $xStr tmpx
		redimension/n=(numpnts($xStr)-1) tmpx
		SIS_graphToFront("WF_FIT","W_ExtractedCol","tmpx")
		SetAxis/A bottom
		// loop
		i=fromFrame
		variable j=0
		variable incr=1
		if (fromFrame>toFrame)
			incr=-1
		endif
		String messageStr
		for (j=min(fromFrame,toFrame);j<=max(fromFrame,toFrame);j+=1)
			imageTransform/G=(i) getCol $wfStr
			messageStr=num2str(abs(i-fromframe))+" out of "+num2str(max(fromFrame,toFrame)-min(fromFrame,toFrame))
			TextBox/C/N=text0/A=RT messageStr
			DoUpdate
			if (shirley == 2)
				FIT_removeShirleyFromProfile("W_ExtractedCol","tmpx",npntsbkg)
			endif
			DoUpdate
			// do the fit
			if (mode == 1)
				if (strsearch(dsf_text[6],"Doublet",0)==-1)
					FuncFit/H=(tmpHoldStr)/Q/G/NTHR=0 FIT_multiPeakDS w_dsf_coef W_ExtractedCol[fromhere,tothere] /X=tmpx /D /R /C=dsf_fit_cnstr
				else
					FuncFit/H=(tmpHoldStr)/Q/G/NTHR=0 FIT_multiDoubletDS w_dsf_coef W_ExtractedCol[fromhere,tothere] /X=tmpx /D /R /C=dsf_fit_cnstr
				endif
			endif
			if (mode == 2)
				FuncFit/H=(tmpHoldStr)/Q/G/NTHR=0 FIT_multiPeakGauss w_dsf_coef W_ExtractedCol[fromhere,tothere] /X=tmpx /D /R /C=dsf_fit_cnstr
			endif
			if (mode == 3)
				FuncFit/H=(tmpHoldStr)/Q/G/NTHR=0 FIT_multiPeakLor w_dsf_coef W_ExtractedCol[fromhere,tothere] /X=tmpx /D /R /C=dsf_fit_cnstr
			endif
			if (mode==4)
				Wavestats W_ExtractedCol
				FuncFit/H=(tmpHoldStr)/Q/G/NTHR=0 FIT_sigmoid w_dsf_coef W_ExtractedCol[fromhere,tothere] /X=tmpx /D /R /C=dsf_fit_cnstr
				FuncFit/H=(tmpHoldStr)/Q/G/NTHR=0 FIT_stepFunction w_dsf_coef W_ExtractedCol[fromhere,tothere] /X=tmpx /D /R /C=dsf_fit_cnstr
			endif
			if (mode==5)
				FuncFit/H=(tmpHoldStr)/G/NTHR=0 FIT_sigmoid w_dsf_coef W_ExtractedCol[fromhere,tothere] /X=tmpx /D /R /C=dsf_fit_cnstr
			endif
			// store the results using putcol
			imageTransform/G=(i)/D=w_dsf_coef putrow fit_results
			imageTransform/G=(i)/D=w_sigma putrow fit_errors
			i+=incr
		endfor
		//edit results
		DoWindow/K WF_FIT
		String tableStr="RT"+dfStr
		SIS_tableToFront(tableStr,"fit_results")	
	endif
end

Function FIT_extract_results()
	// data folder
	String dfStr=SIS_getDataFolderName()
	SetDataFolder root:$(dfStr)
	// waves
	String winStr=SIS_getWinName()
	// globals
	wave dsf_coef=root:$(dfStr):dsf_coef
	wave dsf_hold=root:$(dfStr):dsf_hold
	//NVAR dsf_npeaks=root:$(dfStr):dsf_npeaks
	SVAR holdStr=root:$(dfStr):holdStr	
	wave fit_results=root:$(dfStr):fit_results	
	// input
	String xStr
	Prompt xStr, "enter x-axis wave",popup WaveList("WF*_*Q",";","")
	Variable parameter=5
	Prompt parameter, "Parameter", popup "Lorentzian width; Asymmetry; Gaussian width; peak intensity; peak position"
	doPrompt "series fit extract parameter", xStr,parameter
	if (V_flag ==0)
	// extract and display
	variable np=dimsize($xStr,0)
	variable i=0,j=0
	String tmpStr=""
	String graphStr="FR"+dfStr
	for (i=1;i<=dsf_coef[0];i+=1)
		j=5*i+parameter-1
		tmpStr=graphStr+"_"+num2Str(j)
		print tmpStr
		make/N=(np)/o $tmpStr=NaN
		wave w=$tmpStr
		w=fit_results[p][j]
		String axis=FIT_graphToFront(graphStr,tmpStr,xStr,parameter)
		Label $axis "\\F'Arial'\\Z18Intensity (arb. u.)"
		ModifyGraph lblPosMode=1
		ModifyGraph freePos($axis)=0
		Label bottom "\\F'Arial'\\Z18 x scale (x units)"
		// Label bottom "\\F'Arial'\\Z18Coverage (ps ML)"
		ModifyGraph fSize=12
		if (parameter<4)
			//ModifyGraph axisEnab($axis)={0,0.3}
			Label $axis "\\F'Arial'\\Z18FWHM (eV)"
		elseif (parameter==4)
			//ModifyGraph axisEnab($axis)={0.7,1}
			Label $axis "\\F'Arial'\\Z18Intensity (arb. u.)"
		elseif (parameter==5) 
			//ModifyGraph axisEnab($axis)={0.35,0.65}
			Label $axis "\\F'Arial'\\Z18Peak Position (eV)"
		endif
	endfor
	endif
End

Function FIT_AllWavesToBinding()
	variable keFE=250.79
	Prompt keFE, "kinetic energy of FE"
	String pStr
	Prompt pStr, "enter p wave",popup WaveList("*_P", ";","WIN:")
	doPrompt "operations input" pStr,keFE
	if (V_flag ==0)
		print "ok"
		String wavelistStr = WaveList("fit*", ";","WIN:") 
		variable n=ItemsInList(wavelistStr) 
		variable m=0
		String fitwavenameStr = ""
		variable xright=0
		variable xleft=0
		wave pw=$pStr
		pw-=keFE
		pw*=-1
		do
			fitwavenameStr=StringFromList(m, wavelistStr) 
			wave wfit = $fitwavenameStr
			xleft=keFE-leftx(wfit)
			xright=keFE-rightx(wfit)
			print fitwavenameStr,xleft,xright
			SetScale x xleft,xright,"", wfit
			m+=1
		while(m < n)
		Label bottom "\\F'Arial'\\Z18Binding energy (eV)"
	endif
end

Function FIT_RescaleAllWaves()
	variable scalingFactor=1
	Prompt scalingFactor, "scaling factor"
	doPrompt "Rescale waves" scalingFactor
	if (V_flag ==0)
		String wavelistStr = WaveList("!*P", ";","WIN:") 
		variable n=ItemsInList(wavelistStr) 
		variable m=0
		String wavenameStr = ""
		do
			wavenameStr=StringFromList(m, wavelistStr) 
			wave w = $wavenameStr
			w*=scalingFactor
			m+=1
		while(m < n)
	
	endif
end


Function FIT_findDiracPoint()
	// data folder
	String dfStr=SIS_getDataFolderName()
	SetDataFolder root:$(dfStr)
	// input
	String y1Str
	Prompt y1Str, "enter first wave wave",popup WaveList("*_9",";","")
	String y2Str
	Prompt y2Str, "enter first wave wave",popup WaveList("*_14",";","")
	String xStr
	Prompt xStr, "enter first wave wave",popup WaveList("*_Q",";","")
	doprompt "calculate dirca Point" y1Str,y2Str,xStr
	if (V_Flag==0)
		make/n=2/o W_coef,W_sigma,W_ParamConfidenceInterval
		wave W_coef,W_sigma,W_ParamConfidenceInterval
		variable c1, c2
		CurveFit/NTHR=0/ODR=1/Q/G line  $y2Str[pcsr(A),pcsr(B)] /X=$xStr /D 
		variable a0_2=W_coef[0]
		variable a1_2=W_coef[1]
		duplicate/o $y2Str diff
		duplicate/o $xStr xx
		diff=diff[p]-(a0_2+a1_2*xx[p])
		Wavestats/R=(pcsr(A),pcsr(B))/Q diff
		diff=V_rms
		CurveFit/NTHR=0/Q line  $y2Str[pcsr(A),pcsr(B)] /X=$xStr /D /W=diff /I=1 /D /F={0.683000, 4}
		a0_2=W_coef[0]
		a1_2=W_coef[1]
		String fitStr="fit_"+y2Str
		wavestats $xStr
		variable xl=V_min
		variable xr=V_max
		SetScale/I x xl,xr,"",$fitStr
		wave wf=$fitStr
		wf=a0_2+a1_2*x
		// calc error
		variable sig0_2=W_ParamConfidenceInterval[0]
		variable sig1_2=W_ParamConfidenceInterval[1]
		CurveFit/NTHR=0/ODR=1/Q/G line  $y1Str[pcsr(A),pcsr(B)] /X=$xStr /D 
		variable a0_1=W_coef[0]
		variable a1_1=W_coef[1]
		// make/N=2
		duplicate/o $y1Str diff
		duplicate/o $xStr xx
		diff=diff[p]-(a0_1+a1_1*xx[p])
		Wavestats/R=(pcsr(A),pcsr(B))/Q diff
		diff=V_rms 
		CurveFit/NTHR=0/Q line $y1Str[pcsr(A),pcsr(B)] /X=$xStr /D /W=diff /I=1 /D/F={0.683000, 4}
		a0_1=W_coef[0]
		a1_1=W_coef[1]
		fitStr="fit_"+y1Str
		SetScale/I x xl,xr,"",$fitStr
		wave wf=$fitStr
		wf=a0_1+a1_1*x
		variable sig0_1=W_ParamConfidenceInterval[0]
		variable sig1_1=W_ParamConfidenceInterval[1]
		variable E_d=(a0_2-a0_1)/(a1_1-a1_2)
		variable hplanck=6.626007E-34
		variable eVJ=1.6021765E-19
		variable VF=1/(hplanck*(abs(a1_1)+abs(a1_2))*0.5*0.0028*0.5*1E10/eVJ) //0.0028 for Ir001
		variable err_E_d=sqrt(sig1_2^2+sig1_1^2)/abs(a1_1-a1_2) // + abs(a0_2-a0_1)*(sqrt(sig1_2^2+sig1_1^2))/(a1_1-a1_2)^2
		string  msgStr="The Dirac point is located at: "+num2Str(E_d)+" pm "+num2Str(err_E_d)+"\r Fermi velocity is: "+num2Str(VF)
		make/N=2/O edw
		edw=E_d
		SetScale/I x -1,1,"",edw
		TextBox/C/N=text1/A=RB msgStr
		print a1_1,a1_2,VF
	endif
end

Function/S FIT_graphToFront(graphNameStr,waveNameStr,wavexNameStr,axis)
	String graphNameStr
	String waveNameStr
	String wavexNameStr
	variable axis
	String axisStr="Left"+num2Str(axis)
	graphNameStr= SIS_removeQuotes(graphNameStr)
	Dowindow $graphNameStr
	if (V_flag==0)
		Display
		DoWindow/C $graphNameStr
		if (axis==3 || axis==4)
		appendToGraph/L=$(axisStr) $waveNameStr vs $wavexNameStr
		else
		appendToGraph/R=$(axisStr) $waveNameStr vs $wavexNameStr
		endif
		ModifyGraph freePos($axisStr)=0
		Legend/C/N=text0/A=LT
		ModifyGraph rgb($waveNameStr)=(abs(enoise(65280)),abs(enoise(65280)),abs(enoise(65280)))
		ModifyGraph mirror($axisStr)=1
		ModifyGraph width=240,height={Aspect,1}
	else
		DoWindow/F $graphNameStr
		String empty=WaveList(waveNameStr, ";", "WIN:"+graphNameStr)
		if (strlen(empty)==0)
			if (axis==3 || axis==4)
				appendToGraph/L=$(axisStr) $waveNameStr vs $wavexNameStr
			else
				appendToGraph/R=$(axisStr) $waveNameStr vs $wavexNameStr
			endif
			ModifyGraph rgb($waveNameStr)=(abs(enoise(65280)),abs(enoise(65280)),abs(enoise(65280)))
			ModifyGraph msize($waveNameStr)=2
			ModifyGraph mirror($axisStr)=1
			ModifyGraph nticks(bottom)=4
			ModifyGraph minor(bottom)=1
		endif
	endif
	return axisStr
end

Function FIT_extract_sigmoid_results()
	// datafolder
	String dfStr=SIS_getDataFolderName()
	SetDataFolder root:$(dfStr)
	// waves
	String winStr=SIS_getWinName()
	// globals
	wave dsf_coef=root:$(dfStr):dsf_coef
	wave dsf_hold=root:$(dfStr):dsf_hold
	SVAR holdStr=root:$(dfStr):holdStr	
	wave fit_results=root:$(dfStr):fit_results	
	wave fit_errors=root:$(dfStr):fit_errors	
	// input
	String xStr
	Prompt xStr, "enter x-axis wave",popup WaveList("*",";","")
	Variable parameter=2
	Prompt parameter, "Parameter", popup "x0; fwhm; height"
	doPrompt "series fit extract parameter", xStr,parameter
	if (V_flag==0)
		// extract and display
		variable np=dimsize($xStr,0)
		variable i=0,j=0
		String tmpStr=""
		String errStr=""
		parameter+=4
		tmpStr=winStr+"_"+num2Str(parameter)
		errStr=winStr+"_E"+num2Str(parameter)
		make/N=(np)/o $tmpStr=NaN
		make/N=(np)/o $errStr=NaN
		wave w=$tmpStr
		wave ew=$errStr
		w=fit_results[p][parameter]
		ew=fit_errors[p][parameter]
		String graphStr="FR"+dfStr
		SIS_graphToFront(graphStr,tmpStr,xStr)
		if (parameter==5)
			Label left "\\F'Arial'\\Z18position (arb. u.)"
		endif
		if (parameter==6)
			Label left "\\F'Arial'\\Z18fwhm (arb. u.)"
		endif
		if (parameter==7)
			Label left "\\F'Arial'\\Z18amplitude (arb. u.)"
		endif
		Label bottom "\\F'Arial'\\Z18 x scale (x units)"
		ModifyGraph mode=3,marker=8
		ErrorBars $tmpStr Y,wave=($errStr,$errStr)
		ModifyGraph fSize=12
		// buttons
		ControlBar 21
		button saveButton, proc=SiS_waves2itx, pos={1,1} , size={79,18}, title="save waves" 
		button killButton, proc=PROI_killwaves ,pos={81,1} , size={79,18}, title="kill waves" 
	endif
End

Function FIT_DSsingle_KE(w,x): FitFunc
	Wave w
	Variable x
	// keep for historical reasons: call FIT_BE(abs(w[1]),abs(w[4]),abs(w[2]),abs(w[3]),x)
	//CurveFitDialog/ These comments were created by the Curve Fitting dialog. Altering them will
	//CurveFitDialog/ make the function less convenient to work with in the Curve Fitting dialog.
	//CurveFitDialog/ Equation:
	//CurveFitDialog/ f(x) = sum(i) sum (DS (x(i)-y) * exp () * dy)
	//CurveFitDialog/ End of Equation
	//CurveFitDialog/ Independent Variables 5
	//CurveFitDialog/ x
	//CurveFitDialog/ Coefficients 5 // right order: lorw, alpha, gaussw, ampli, pos
	//CurveFitDialog/ w[0] = lorw
	//CurveFitDialog/ w[1] = alpha
	//CurveFitDialog/ w[2] = gaussw
	//CurveFitDialog/ w[3] = Ampl
	//CurveFitDialog/ w[4] = pos
	// ...
	Variable i=0
	Variable sigma = w[2]/(2*sqrt(2*ln(2)))  
	Variable nsteps = 80
	Variable integral = 0
	Variable start = -4*sigma
	Variable stop = -start
	Variable delta = 2* abs (start) / nsteps
	Variable ds=0,y
	Variable normG = 1/(sqrt(2*Pi)*sigma*2.35482*2)
	do
		ds=0
		y=x-start
		ds=FIT_DS_KE(abs(w[1]),abs(w[0]),w[4],abs(w[3]),y)
		integral+=normG*exp(-(start)^2/(2*sigma^2))*(ds)*delta
		start+=delta 	
	while (start <= stop)
	return integral
end

Function FIT_DSdoublet_KE(w,x): FitFunc
	Wave w
	Variable x
	// keep for historical reasons: call FIT_BE(abs(w[1]),abs(w[4]),abs(w[2]),abs(w[3]),x)
	//CurveFitDialog/ These comments were created by the Curve Fitting dialog. Altering them will
	//CurveFitDialog/ make the function less convenient to work with in the Curve Fitting dialog.
	//CurveFitDialog/ Equation:
	//CurveFitDialog/ f(x) = sum(i) sum (DS (x(i)-y) * exp () * dy)
	//CurveFitDialog/ End of Equation
	//CurveFitDialog/ Independent Variables 7
	//CurveFitDialog/ x
	//CurveFitDialog/ Coefficients 7 // right order: lorw, alpha, gaussw, ampli, pos
	//CurveFitDialog/ w[0] = lorw
	//CurveFitDialog/ w[1] = alpha
	//CurveFitDialog/ w[2] = gaussw
	//CurveFitDialog/ w[3] = A1
	//CurveFitDialog/ w[4] = pos1
	//CurveFitDialog/ w[5] = A2
	//CurveFitDialog/ w[6] = deltapos2
	// ...
	Variable i=0
	Variable sigma = w[2]/(2*sqrt(2*ln(2)))  
	Variable nsteps = 80
	Variable integral = 0
	Variable start = -4*sigma
	Variable stop = -start
	Variable delta = 2* abs (start) / nsteps
	Variable ds=0,y
	Variable normG = 1/(sqrt(2*Pi)*sigma*2.35482*2)
	do
		ds=0
		y=x-start
		ds=FIT_DS_KE(abs(w[1]),abs(w[0]),w[4],abs(w[3]),y)+FIT_DS_KE(abs(w[1]),abs(w[0]),w[4]+w[6],abs(w[5]),y)
		integral+=normG*exp(-(start)^2/(2*sigma^2))*(ds)*delta
		start+=delta 	
	while (start <= stop)
	return integral
end

// Binding energy (BE are defined positive)
Function FIT_DS_BE(alpha,gammaL,pos,Ampli,x) 
	Variable alpha,gammaL,pos,Ampli
	Variable x
	gammaL/=2
	alpha=abs(alpha)
	return abs(Ampli) * exp (gammln(1-alpha))*cos(0.5*Pi*alpha+(1-alpha)*atan((pos-x)/gammaL))/((x-pos)^2+gammaL^2)^(0.5*(1-alpha))
End

// kinetic energy
Function FIT_DS_KE(alpha,gammaL,pos,Ampli,x) 
	Variable alpha,gammaL,pos,Ampli
	Variable x
	gammaL/=2
	alpha=abs(alpha)
	return abs(Ampli) * exp (gammln(1-alpha,1e-7))*cos(0.5*Pi*alpha-(1-alpha)*atan((pos-x)/gammaL))/((x-pos)^2+gammaL^2)^(0.5*(1-alpha))  // DS paper
End	

Macro FIT_MyStyle()
	SetAxis bottom 62,48
	SetAxis film 0,100
	SetAxis typeB 0,100
	SetAxis typeA 0,150
end